package test;

import java.util.concurrent.TimeUnit;

import javax.interceptor.AroundInvoke;
import javax.interceptor.InvocationContext;
import javax.persistence.Cache;
import javax.persistence.EntityManagerFactory;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceUnit;

import org.hibernate.Session;
import org.hibernate.stat.Statistics;

/**
 * Interceptor for persistence metrics.
 */
public class PersistenceMetricsInterceptor {

    @PersistenceContext(unitName = "testPU")
    private Session session;

    @PersistenceUnit(unitName = "testPU")
    private EntityManagerFactory emf;

    private static boolean active = true;

    @AroundInvoke
    public Object intercept(final InvocationContext ctx) throws Exception {

        final long startTime = System.nanoTime();
        try {
            return ctx.proceed();
        } finally {
            if (active) {
                // duration
                final long durationInNs = System.nanoTime() - startTime;
                final long durationInMs = TimeUnit.MILLISECONDS.convert(durationInNs, TimeUnit.NANOSECONDS);
                final String durationMessage =
                        String.format("*** PersistenceMetricsInterceptor method [%s], processing time = [%s] ms", ctx.getMethod()
                                .getName(), durationInMs);
                System.out.println(durationMessage);

                this.logCacheStatistics();
                this.logCacheContents();
            }
        }
    }

    private void logCacheStatistics() {
        final Statistics statistics = this.session.getSessionFactory().getStatistics();

        // second level cache
        final long secondLevelCachePutCount = statistics.getSecondLevelCachePutCount();
        final double secondLevelCacheHitCount = statistics.getSecondLevelCacheHitCount();
        final double secondLevelCacheMissCount = statistics.getSecondLevelCacheMissCount();
        final double secondLevelCacheHitRatio = secondLevelCacheHitCount / (secondLevelCacheHitCount + secondLevelCacheMissCount);
        final String secondLevelCacheLogMessage =
                String.format("SECOND LEVEL CACHE has [HitRatio=%s] with [PutCount=%s]", secondLevelCacheHitRatio,
                        secondLevelCachePutCount);
        System.out.println(secondLevelCacheLogMessage);

        // query cache hits
        final long queryCachePutCount = statistics.getQueryCachePutCount();
        final double queryCacheHitCount = statistics.getQueryCacheHitCount();
        final double queryCacheMissCount = statistics.getQueryCacheMissCount();
        final double queryCacheHitRatio = queryCacheHitCount / (queryCacheHitCount + queryCacheMissCount);
        final String queryCacheLogMessage =
                String.format("QUERY CACHE has [HitRatio=%s] with [PutCount=%s]", queryCacheHitRatio, queryCachePutCount);
        System.out.println(queryCacheLogMessage);

        // general cache infos
        final long entityLoadCount = statistics.getEntityLoadCount();
        System.out.println("Entity load count: " + entityLoadCount);

    }

    private void logCacheContents() {

        final Cache cache = this.emf.getCache();
        final String cacheContentsLog = String.format("------------> cache %s", cache);
        System.out.println(cacheContentsLog);

    }
}
